/*******************************************************************************
 *  Licensed Materials - Property of IBM
 * (C) Copyright IBM Corp.  2001, 2006.  All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.ccl.mapping.xsd;

import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.EcoreFactory;
import org.eclipse.emf.ecore.util.ExtendedMetaData;
import org.eclipse.emf.ecore.xml.namespace.XMLNamespacePackage;

import org.eclipse.xsd.*;
import org.eclipse.xsd.util.XSDConstants;

public class XSDEcoreBuilder extends org.eclipse.xsd.ecore.XSDEcoreBuilder
{
  public EPackage getEPackage(XSDSchema xsdSchema)
  {                    
      if (xsdSchema == null)
          xsdSchema = rootSchema;

      if (!xsdSchemas.contains(xsdSchema))
      {
          xsdSchemas.add(xsdSchema);
          addInput(xsdSchema);
          validate(xsdSchema);
      }

      String targetNamespace = xsdSchema.getTargetNamespace();
      EPackage ePackage = (EPackage) targetNamespaceToEPackageMap.get(targetNamespace);
      if (ePackage == null)
      {
          if (XMLNamespacePackage.eNS_URI.equals(targetNamespace) && xsdSchemas.indexOf(xsdSchema) != 0)
          {
              ePackage = XMLNamespacePackage.eINSTANCE;
              for (Iterator i = xsdSchema.getAttributeDeclarations().iterator(); i.hasNext();)
              {
                  XSDAttributeDeclaration xsdAttributeDeclaration = (XSDAttributeDeclaration) i.next();
                  if (!XSDConstants.isSchemaInstanceNamespace(xsdAttributeDeclaration.getTargetNamespace()))
                  {
                      EStructuralFeature eStructuralFeature = ExtendedMetaData.INSTANCE.getAttribute(XMLNamespacePackage.eNS_URI,
                              xsdAttributeDeclaration.getName());
                      map(xsdAttributeDeclaration, eStructuralFeature);
                      XSDTypeDefinition xsdTypeDefinition = xsdAttributeDeclaration.getAnonymousTypeDefinition();
                      if (xsdTypeDefinition != null)
                      {
                          map(xsdTypeDefinition, eStructuralFeature.getEType());
                      }
                  }
              }
          }
          else
          {
              ePackage = EcoreFactory.eINSTANCE.createEPackage();
              setAnnotations(ePackage, xsdSchema);
              addOutput(ePackage);
              if (targetNamespace == null)
              {
                  if (xsdSchema == null)
                  {
                      xsdSchema = rootSchema;
                  }
                  ePackage.setName(validName(xsdSchema.eResource().getURI().trimFileExtension().lastSegment(), true));
                  ePackage.setNsURI(xsdSchema.eResource().getURI().toString());

                  // Also register against the nsURI for the case that the target namespace is null.
                  //
                  // extendedMetaData.putPackage(ePackage.getNsURI(), ePackage);
              }
              else
              {
                  URI uri = URI.createURI(targetNamespace);
                  List parsedName;
                  if (uri.isHierarchical())
                  {
                      String host = uri.host();
                      if (host != null && host.startsWith("www.")) //$NON-NLS-1$
                      {
                          host = host.substring(4);
                      }
                      parsedName = parseName(host, '.');
                      Collections.reverse(parsedName);
                      if (!parsedName.isEmpty())
                      {
                          parsedName.set(0, ((String) parsedName.get(0)).toLowerCase());
                      }

                      parsedName.addAll(parseName(uri.trimFileExtension().path(), '/'));
                  }
                  else
                  {
                      String opaquePart = uri.opaquePart();
                      int index = opaquePart.indexOf(":"); //$NON-NLS-1$
                      if (index != -1 && "urn".equalsIgnoreCase(uri.scheme())) //$NON-NLS-1$
                      {
                          parsedName = parseName(opaquePart.substring(0, index), '-');
                          if (parsedName.size() > 0 && DOMAINS.contains(parsedName.get(parsedName.size() - 1)))
                          {
                              Collections.reverse(parsedName);
                              parsedName.set(0, ((String) parsedName.get(0)).toLowerCase());
                          }
                          parsedName.addAll(parseName(opaquePart.substring(index + 1), '/'));
                      }
                      else
                      {
                          parsedName = parseName(opaquePart, '/');
                      }
                  }

                  StringBuffer qualifiedPackageName = new StringBuffer();
                  for (Iterator i = parsedName.iterator(); i.hasNext();)
                  {
                      String packageName = (String) i.next();
                      if (packageName.length() > 0)
                      {
                          if (qualifiedPackageName.length() > 0)
                          {
                              qualifiedPackageName.append('.');
                          }
                          qualifiedPackageName.append(validName(packageName, false));
                      }
                  }
                  ePackage.setName(qualifiedPackageName.toString());
                  ePackage.setNsURI(targetNamespace);
              }

              String nsPrefix = null;
              if (targetNamespace != null)
              {
                Map prefixToNamespaceMap = xsdSchema.getQNamePrefixToNamespaceMap();
                if ((prefixToNamespaceMap != null) && (prefixToNamespaceMap.containsValue(targetNamespace)))
                {
            	  for (Iterator i = prefixToNamespaceMap.keySet().iterator(); i.hasNext();)
            	  {
            		Object key = i.next();
            		Object value = prefixToNamespaceMap.get(key);
            		if ((key instanceof String) && (value instanceof String))
            		{
            		  if (targetNamespace.equals((String)value))
            		  {
            			nsPrefix = (String)key;
            			ePackage.setNsPrefix(nsPrefix);
            		  }
            		}
            	  }	
            	}
              }
              if (nsPrefix == null)
              {
                nsPrefix = ePackage.getName();
                int index = nsPrefix.lastIndexOf('.');
                ePackage.setNsPrefix(index == -1 ? nsPrefix : nsPrefix.substring(index + 1));
              }
              extendedMetaData.setQualified(ePackage, targetNamespace != null);
              extendedMetaData.putPackage(targetNamespace, ePackage);
          }

          targetNamespaceToEPackageMap.put(targetNamespace, ePackage);
      }

      return ePackage;
  }

}
