/*******************************************************************************
 *  Licensed Materials - Property of IBM
 * (C) Copyright IBM Corp.  2001, 2006.  All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.ccl.mapping.xsd;

import java.util.List;
import org.eclipse.emf.ecore.EObject;

import com.ibm.ccl.mapping.CustomFunctionRefinement;
import com.ibm.ccl.mapping.Mapping;
import com.ibm.ccl.mapping.MappingDeclaration;

import com.ibm.ccl.mapping.MappingRoot;
import com.ibm.ccl.mapping.codegen.MappingHandler;

/**
 * A handler interface to access xml to xml mapping information.
 * @author lauzond
 */

public interface XSDEcoreMappingHandler extends MappingHandler
{

	/**
	 * Returns the list of attributes of the specified XSD element.
	 * @param parent an XSD element.
	 * @return the list of attributes for the specified XSD element.
	 */
	public List getAttributes(EObject parent);
	
	/**
	 * Returns the list of child elements for the specified XSD element.
	 * @param parent an XSD element.
	 * @return the list of attributes for the specified XSD element.
	 */
	public List getElements(EObject parent);
	
  /**
   * Returns true if the specified eobject represents an XSD element.
   * @param eobject an EObject.
   * @return true if the specified eobject represents an XSD element.
   */
	public boolean isElement(EObject eobject);

  /**
   * Returns the root mapping defined within this mapping root.
   * There may be more than a single mapping declaration, in which
   * case, it is the declaration that is not referenced. However, if 
   * there are 2 non-referenced mapping declarations, the first one
   * is chosen.
   * 
   * @param mappingRoot the mapping root.
   * @return the root mapping defined within this mapping root.
   */
	public Mapping getRootMapping(MappingRoot mappingRoot);
  
  /**
   * Return true if the mapping declaration is referenced within
   * the mapping root.
   * 
   * @param mappingRoot the mapping root.
   * @param mappingDeclaration the mapping declaration.
   * @return
   */
	public boolean isReferenced(MappingRoot mappingRoot, MappingDeclaration mappingDeclaration);
  
  /**
   * Returns true if the mapping contains at least one type.
   * @param rootMapping a mapping
   * @return true if the mapping is of at least one type.
   */
    public boolean isTypeMapping(Mapping mapping);

   /**
	* Returns true of the EObject represents an XML Schema wildcard.
	* @param eobject an XML Schema object.
	* @return true of the EObject represents an XML Schema wildcard.
	*/
    public boolean isWildcard(EObject eobject);

   /**
	* Returns true if the given mapping has no inputs.
	* @param mapping
	* @return true if the given mapping has no inputs.
	*/
    public boolean isTargetMappingOnly(Mapping mapping);

    /**
     * 
     * @param refinement
     * @return
     * @deprecated use ModelUtils.isExternalCustomDefinition(CustomFunctionRefinement refinement).
     */
    public boolean isExternalCustomDefinition(CustomFunctionRefinement refinement);

    /**
     * 
     * @param mapping
     * @return
     * @deprecated use ModelUtils.isMove(Mapping mapping).
     */
public boolean isMove(Mapping mapping);
}