/*******************************************************************************
 *  Licensed Materials - Property of IBM
 * (C) Copyright IBM Corp.  2001, 2006.  All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.ccl.mapping.xsd;

import java.util.Iterator;

import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EClassifier;
import org.eclipse.emf.ecore.EDataType;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.util.ExtendedMetaData;


public class XSDEcoreVisitor
{
  public XSDEcoreVisitor()
  {
  }
  
  public void visitSchema(EPackage schema)
  {
    EClass documentRoot = ExtendedMetaData.INSTANCE.getDocumentRoot(schema);
    if (documentRoot != null)
    {
      // visit the XSD global elements
	  for (Iterator i = ExtendedMetaData.INSTANCE.getElements(documentRoot).iterator(); i.hasNext();)
      {
        visitElement((EStructuralFeature)i.next());
      }

	  // visit the XSD global attributes
      for (Iterator i = ExtendedMetaData.INSTANCE.getAttributes(documentRoot).iterator(); i.hasNext();)
      {
        visitAttribute((EStructuralFeature)i.next());
      }

      for (Iterator i = schema.getEClassifiers().iterator(); i.hasNext();)
      {
        visitType((EClassifier)i.next());
      }
    }
  }
  
  public void visitElement(EStructuralFeature element)
  {
	  EClassifier eClassifier = element.getEType();
	  if (eClassifier != null)
	  {
		visitType(eClassifier);
	  }
  }
  
  public void visitAttribute(EStructuralFeature attr)
  {   
  }
  
  public void visitType(EClassifier type)
  {   
	  if (type instanceof EDataType)
	  {
		  visitSimpleType((EDataType)type);
	  }
	  else
	  {
		  visitComplexType((EClass)type);
	  }
  }

  public void visitSimpleType(EDataType simpleType)
  { 
  }

  public void visitComplexType(EClass complexType)
  {
	    // visit the XSD global elements
		for (Iterator i = ExtendedMetaData.INSTANCE.getElements(complexType).iterator(); i.hasNext();)
	    {
	      visitElement((EStructuralFeature)i.next());
	    }

		// visit the XSD global attributes
	    for (Iterator i = ExtendedMetaData.INSTANCE.getAttributes(complexType).iterator(); i.hasNext();)
	    {
	      visitAttribute((EStructuralFeature)i.next());
	    }

  }
}
