/*******************************************************************************
 *  Licensed Materials - Property of IBM
 * (C) Copyright IBM Corp.  2001, 2006.  All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.ccl.mapping.xsd.impl;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EClassifier;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.util.ExtendedMetaData;

import com.ibm.ccl.mapping.CustomFunctionRefinement;
import com.ibm.ccl.mapping.Mapping;
import com.ibm.ccl.mapping.MappingDeclaration;
import com.ibm.ccl.mapping.MappingDesignator;
import com.ibm.ccl.mapping.MappingRoot;
import com.ibm.ccl.mapping.SubmapRefinement;
import com.ibm.ccl.mapping.codegen.impl.MappingHandlerImpl;
import com.ibm.ccl.mapping.codegen.util.ModelUtils;
import com.ibm.ccl.mapping.xsd.XSDEcoreExtendedMetaData;
import com.ibm.ccl.mapping.xsd.XSDEcoreMappingHandler;
import com.ibm.ccl.mapping.xsd.util.XSDEcoreUtils;

public class XSDEcoreMappingHandlerImpl extends MappingHandlerImpl implements	XSDEcoreMappingHandler
{
  /* (non-Javadoc)
   * @see com.ibm.etools.msl.MappingHandler#getMappingInfo(com.ibm.etools.msl.Mapping, org.eclipse.emf.ecore.EObject)
   */
  public int getMappingInfo(Mapping baseMapping, EObject target)
  {
    int result = NO_MAPPING;
    if (baseMapping != null)
    {	
      EObject commonAncestor = getPrimaryTarget(baseMapping);
      List mappings = ModelUtils.getInputMappings(baseMapping, commonAncestor, target);   
      if ((mappings == null) || (mappings.isEmpty()))
      {
        if (isContentMapped(getNestedMappings(baseMapping), commonAncestor, target))
        {
    	  result = NO_MAPPING_WITH_MAPPED_CONTENT; 
        }
      }
      else if (mappings.size() == 1)
      {
        List inputs = ((Mapping)mappings.get(0)).getInputs();
        if (inputs.size() > 1)
        {
    	  result = SINGLE_MAPPING_WITH_MULTIPLE_INPUTS;
        }
        else
        {
    	  result = SINGLE_MAPPING;
        }
      }
      else if (mappings.size() > 1)
      {
        result = MULTIPLE_MAPPINGS;
      }
    }
    return result;
  }

	  
  /* (non-Javadoc)
   * @see com.ibm.etools.msl.MappingHandler#getMappingInfo(java.util.List, org.eclipse.emf.ecore.EObject)
   */
  public int getMappingInfo(List nestedMappings, EObject target)
  {
    List mappings = null;
    EObject commonAncestor = null;
	if ((nestedMappings != null) && (!nestedMappings.isEmpty()))
	{
	  int size = nestedMappings.size()-1;
	  commonAncestor = getPrimaryTarget((Mapping)nestedMappings.get(size));
	  for (int i = size; i>=0; i--)
	  {
	    Mapping contextMapping = (Mapping)nestedMappings.get(i);
	    mappings = ModelUtils.getInputMappings(contextMapping, commonAncestor, target);
	    if ((mappings != null) && (!mappings.isEmpty()))
	    {
	      break;
	    }
	  }
	}
	int result = NO_MAPPING;
	if ((mappings == null) || (mappings.isEmpty()))
	{
	  int size = nestedMappings.size()-1;
	  for (int i = size; i>=0; i--)
	  {
	    Mapping contextMapping = (Mapping)nestedMappings.get(i);
	    if (isContentMapped(getNestedMappings(contextMapping), commonAncestor, target))
	    {
	      result = NO_MAPPING_WITH_MAPPED_CONTENT;
	      break;
	    }
	  }
	}
	else if (mappings.size() == 1)
	{
	  List inputs = ((Mapping)mappings.get(0)).getInputs();
	  if (inputs.size() > 1)
	  {
	   	result = SINGLE_MAPPING_WITH_MULTIPLE_INPUTS;
	  }
	  else
	  {
	    result = SINGLE_MAPPING;
	  }
	}
	else if (mappings.size() > 1)
	{
	  result = MULTIPLE_MAPPINGS;
    }
    return result;
  }

  /* (non-Javadoc)
   * @see com.ibm.ccl.mapping.xml.MappingXMLHandler#getAttributes(org.eclipse.emf.ecore.EObject)
   */
  public List getAttributes(EObject parent)
  {
	  List result = new ArrayList();
	  EClass eClass =  getEClass(parent);
	  
	  if (eClass != null)
	  {
	    List features = eClass.getEAllStructuralFeatures();
	    if (features != null)
	    {
		  for (Iterator i = features.iterator(); i.hasNext();)
		  {
		    EStructuralFeature child = (EStructuralFeature)i.next();
		    if ((!isElement(child)) && (!XSDEcoreUtils.isSimpleContent(child)))
		    {
		      result.add(child);
		    }
		  }
	    }
	  }
  	return result;
  }

  
  /* (non-Javadoc)
   * @see com.ibm.ccl.mapping.xml.MappingXMLHandler#getElements(org.eclipse.emf.ecore.EObject)
   */
  public List getElements(EObject parent)
  {
    List result = new ArrayList();
	EClass eClass = getEClass(parent);
    
    if (eClass != null)
    {
	  List features = eClass.getEAllStructuralFeatures();
	  if (features != null)
	  {
	    for (Iterator i = features.iterator(); i.hasNext();)
	    {
	      EStructuralFeature child = (EStructuralFeature)i.next();
	      if ((isElement(child)) && (!XSDEcoreUtils.isMixedContent(child)))
	      {
	        result.add(child);
	      }
	    }
	  }
    }
	return result;
  }

  
  private EClass getEClass(EObject eobject) 
  {
	 EClass result = null;
	 if (eobject instanceof EStructuralFeature)
	 {	
	   EClassifier eClassifier = ((EStructuralFeature)eobject).getEType();
	   if (eClassifier instanceof EClass)
	   {
	     result = (EClass)eClassifier;
	   }
	 }
	 else if (eobject instanceof EClass)
	 {
	   result = (EClass)eobject;
     }
	return result;
}


/* (non-Javadoc)
   * @see com.ibm.ccl.mapping.xml.MappingXMLHandler#isElement(org.eclipse.emf.ecore.EObject)
   */
  public boolean isElement(EObject eobject)
  {
    boolean result = false;
    if (eobject instanceof EStructuralFeature)
    {
      switch (XSDEcoreExtendedMetaData.INSTANCE.getFeatureKind((EStructuralFeature)eobject))
      {
        case ExtendedMetaData.ELEMENT_FEATURE:
        case ExtendedMetaData.ELEMENT_WILDCARD_FEATURE:
        case ExtendedMetaData.GROUP_FEATURE:
        {
          result = true;
          break;
        }
      }
    }
    return result;
  }


  /* (non-Javadoc)
   * @see com.ibm.ccl.mapping.xsd.XSDEcoreMappingHandler#getRootMapping(com.ibm.ccl.mapping.MappingRoot)
   */
  public Mapping getRootMapping(MappingRoot mappingRoot) 
  {
	Mapping result = null;
	for (Iterator i = mappingRoot.getNested().iterator(); i.hasNext();)
	{
	  Object obj = i.next();
	  if (obj instanceof MappingDeclaration)
	  if (!isReferenced(mappingRoot, (MappingDeclaration)obj))
	  {
		result = (MappingDeclaration)obj;
		break;
	  }
	}
	return result;
  }


  /* (non-Javadoc)
   * @see com.ibm.ccl.mapping.xsd.XSDEcoreMappingHandler#isReferenced(com.ibm.ccl.mapping.MappingRoot, com.ibm.ccl.mapping.MappingDeclaration)
   */
  public boolean isReferenced(MappingRoot mappingRoot, MappingDeclaration mappingDeclaration) 
  {
    boolean result = false;
    try
    {
      if ((mappingRoot != null) && (mappingDeclaration != null))
      {
   	    for (Iterator i = mappingRoot.getNested().iterator(); i.hasNext();)
        {
          Object obj = i.next();
          if (obj instanceof MappingDeclaration)
          {
            if (obj != mappingDeclaration)
            {
        	  if (isReferenced((Mapping)obj, mappingDeclaration))
        	  {
        	    result = true;
        	    break;
    	      }
    	    }
          }
        }
      }
    }
    catch (Exception e) {}
	return result;
  }


  protected boolean isReferenced(Mapping mapping, MappingDeclaration reference) 
  {
    boolean result = false;
	try
	{
	  if ((mapping != null) && (reference != null))
	  {
	    SubmapRefinement submap = ModelUtils.getSubmapRefinement(mapping);
	    if (submap != null)
	    {
	      if (submap.getRef() == reference)
	      {
	        result = true;
	      }
	    }
	    if (!result)
	    {
		  // check the nested mappings
	      for (Iterator i = mapping.getNested().iterator(); i.hasNext();)
	      {
	        Object obj = i.next();
	        if (obj instanceof Mapping)
	        {
	          if (obj != mapping)
	          {
	            if (isReferenced((Mapping)obj, reference))
	            {
	              result = true;
	              break;
	    	    }
	          } 
	        }
	      }
	    }
	  }
	}
	catch (Exception e) {}
	return result;
  }


/* (non-Javadoc)
   * @see com.ibm.ccl.mapping.xsd.XSDEcoreMappingHandler#isTypeMapping(com.ibm.ccl.mapping.Mapping)
   */
  public boolean isTypeMapping(Mapping mapping) 
  {
	boolean isTypeMapping = false;
	if (mapping != null)
	{
	  for (Iterator i = mapping.getInputs().iterator(); i.hasNext();)
	  {
		isTypeMapping = isType((MappingDesignator)i.next());
		if (isTypeMapping)
		  break;
	  }
	  if (!isTypeMapping)
	  {
		for (Iterator i = mapping.getOutputs().iterator(); i.hasNext();)
		{
		  isTypeMapping = isType((MappingDesignator)i.next());
		  if (isTypeMapping)
			break;
		 } 
	  }
	}
	return isTypeMapping;
  }
  
  public boolean isContentMapped(List nestedMappings, EObject ancestor, EObject target)
  {
	if ((nestedMappings != null) && (target != null))
	{
      for (Iterator i = nestedMappings.iterator(); i.hasNext();)
      {
    	Mapping mapping = (Mapping)i.next();
        MappingDesignator designator = getPrimaryTargetDesignator(mapping);

        if (ModelUtils.isCommonAncestor(designator, target) && ((ancestor == null) || ModelUtils.isCommonAncestor(designator, ancestor)))
        {
          return true;
     	}
      }
	}
	return false;
  }


  public boolean isWildcard(EObject eobject) 
  {
	if (eobject instanceof EStructuralFeature)
	  return XSDEcoreUtils.isWildcard((EStructuralFeature)eobject);
	else
	  return false;
  }


  /*
   * (non-Javadoc)
   * @see com.ibm.ccl.mapping.xsd.XSDEcoreMappingHandler#isTargetMappingOnly(com.ibm.ccl.mapping.Mapping)
   */
  public boolean isTargetMappingOnly(Mapping mapping) 
  {
	return mapping.getInputs().isEmpty();
  }

  public boolean isMove(Mapping mixedMapping)
  {
	return ModelUtils.hasMoveRefinement(mixedMapping);
  }


  public boolean isExternalCustomDefinition(CustomFunctionRefinement refinement) 
  {
	return ModelUtils.isExternalCustomDefinition(refinement);
  }
}
