/*******************************************************************************
 *  Licensed Materials - Property of IBM
 * (C) Copyright IBM Corp.  2001, 2006.  All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.ccl.mapping.xsd.resources;

import org.eclipse.emf.ecore.resource.Resource.Factory;

/**
 * Implemented by XSD resource providers. XSD resource providers are used by the
 * framework to be able to use other data type specifications (DTD, XML) as
 * inputs/outputs for a mapping.
 */
public interface IXSDResourceProvider
{
  /**
   * The provider name.
   * 
   * @return a String with the provider name.
   */
  String getName();

  /**
   * Provides an instance of the resource factory referenced by this descriptor.
   * 
   * @return a Factory or null if one cannot be instantiated.
   */
  Factory getResourceFactory();

  /**
   * Provides the file extension of the files handeled by the resource factory.
   * 
   * @return the file extension of the files handeled by the resource factory.
   */
  String getFileExtension();

  /**
   * Informs the framework if the provider can be used.
   * 
   * @return true if the provider can be used, false otherwise.
   */
  boolean isAvailable();
}
