/*******************************************************************************
 *  Licensed Materials - Property of IBM
 * (C) Copyright IBM Corp.  2001, 2006.  All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.ccl.mapping.xsd.resources;

import java.util.Iterator;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EClassifier;
import org.eclipse.emf.ecore.EDataType;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.util.BasicExtendedMetaData;

import com.ibm.ccl.mapping.resolvers.TypeHandler;
import com.ibm.ccl.mapping.xsd.Messages;
import com.ibm.ccl.mapping.xsd.util.XSDEcoreUtils;

public class MappingTypeHandler extends TypeHandler 
{
	public String getNameLabel(EStructuralFeature feature) 
	{
		return XSDEcoreUtils.getDisplayName(feature);
	}

  public String getTypeLabel(EClassifier classifier, boolean array) 
	{
		if (classifier == null) 
		{
			return Messages.TYPE_HANDLER_UNRESOLVED_TYPE;
		}

    String displayName = XSDEcoreUtils.getDisplayName(classifier);
    
    if ((!array) || ((displayName != null) && (displayName.trim().length() == 0)))
		{
			return displayName;
		} 
		else 
		{
			return  "[ ] " + displayName;
		}
	}
    
	/**
	 * Answers if classifiers are compatible, i.e. value from the first
	 * classifier is assignable to the value of the second.
	 * 
	 * @param fromClassifier
	 * @param toClassifier
	 * @return boolean
	 */
	public boolean isAssignable(EClassifier fromClassifier,
			EClassifier toClassifier) 
	{
	  boolean result = false;
    
      // Allow any in or any out
    
      if (XSDEcoreUtils.isFeatureMap(fromClassifier) ||
          XSDEcoreUtils.isFeatureMap(toClassifier))
      {
        return true;
      }
    
	  if ((fromClassifier instanceof EDataType)&&
	      (toClassifier instanceof EDataType)) 
	  {
	    result = true;
	  }
	  else if ((fromClassifier instanceof EClass) &&
	      (toClassifier instanceof EClass))
	  {
	    if (isSuperTypeOf((EClass)toClassifier, (EClass)fromClassifier))
	    {
	      result = true;	  
	    }
	    else
	    {
	      result = super.isAssignable(fromClassifier, toClassifier);	
	    }
	  }
	  else
	  {
	    result = super.isAssignable(fromClassifier, toClassifier);
	  }
	  return result;
	}

	/**
	 * Returns true if toClass is a super type of fromClass.
	 * @param toClass 
	 * @param fromClass
	 * @return true if toClass is a super type of fromClass.
	 */
	private boolean isSuperTypeOf(EClass toClass, EClass fromClass) 
	{
	  boolean result = false;
	  if ((toClass != null) && (fromClass != null))
	  {
		EList superTypes = fromClass.getEAllSuperTypes();
		if (superTypes != null)
		{
		  String toName = BasicExtendedMetaData.INSTANCE.getName(toClass);
		  String toNamespace = BasicExtendedMetaData.INSTANCE.getNamespace(toClass);
		  
		  if (toName != null)
		  {
 	        for (Iterator i = superTypes.iterator(); i.hasNext();)
		    {
		      EClass eClass = (EClass)i.next();
		      String name = BasicExtendedMetaData.INSTANCE.getName(eClass);
		      String namespace = BasicExtendedMetaData.INSTANCE.getNamespace(eClass);

		      if (toNamespace != null)
		      {
		        if (toName.equals(name) && toNamespace.equals(namespace))
		        {
		          result = true;
		          break;
		        }
		      }
		      else
		      {
		    	if ((namespace == null) && (toName.equals(name)))
		    	{
			      result = true;
			      break;		    		
		    	}
		      }
		    }
		  }
		}
	  }
	  return result;
	}
}