/*******************************************************************************
 *  Licensed Materials - Property of IBM
 * (C) Copyright IBM Corp.  2001, 2006.  All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.ccl.mapping.xsd.resources.extension;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtensionPoint;
import org.eclipse.core.runtime.IExtensionRegistry;
import org.eclipse.core.runtime.Platform;

import com.ibm.ccl.mapping.xsd.resources.IXSDResourceProvider;

/**
 * This class reads the plugin extension registry and registers each delegating
 * validator descriptor with the delegates registry.
 * 
 * @see ValidatorDelegatesRegistry
 */
class XSDResourceProviderRegistryReader
{
  /**
   * The provider class attribute.
   */
  private static final String CLASS_ATTRIBUTE = "class"; //$NON-NLS-1$

  /**
   * The provider element name.
   */
  private static final String PROVIDER_ELEMENT = "resourceProvider"; //$NON-NLS-1$

  /**
   * The resource provider extension point id.
   */
  private static final String EXTENSION_POINT_ID = "resourceProviders"; //$NON-NLS-1$

  /**
   * Plugin id.
   */
  private static final String PLUGIN_ID = "com.ibm.ccl.mapping.xsd"; //$NON-NLS-1$

  /**
   * The validator registry where the descriptors being read will be placed.
   */
  private XSDResourceProviderRegistry registry;

  /**
   * Constructor.
   * 
   * @param registry the registry where the descriptors being read will be
   *          placed.
   */
  public XSDResourceProviderRegistryReader(XSDResourceProviderRegistry registry)
  {
    this.registry = registry;
  }

  /**
   * Reads a configuration element.
   * 
   * @param element the platform configuration element being read.
   */
  private void readElement(IConfigurationElement element)
  {
    String elementName = element.getName();

    if (elementName.equals(PROVIDER_ELEMENT))
    {
      try
      {
        IXSDResourceProvider resourceProvider = (IXSDResourceProvider) element.createExecutableExtension(XSDResourceProviderRegistryReader.CLASS_ATTRIBUTE);
        
        if (resourceProvider.isAvailable())
        {
          registry.addResourceProvider(resourceProvider);
        }
      }
      catch (CoreException e)
      {
        // TODO Log error.
        System.out.println(e.getMessage());
      }
    }
  }

  /**
   * Read from the extensions registry and parse it.
   */
  void readRegistry()
  {
    IExtensionRegistry pluginRegistry = Platform.getExtensionRegistry();
    IExtensionPoint point = pluginRegistry.getExtensionPoint(PLUGIN_ID, EXTENSION_POINT_ID);

    if (point != null)
    {
      IConfigurationElement[] elements = point.getConfigurationElements();

      for (int index = 0; index < elements.length; index++)
      {
        readElement(elements[index]);
      }
    }
  }
}
