/*******************************************************************************
 *  Licensed Materials - Property of IBM
 * (C) Copyright IBM Corp.  2001, 2006.  All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.ccl.mapping.xsd.util;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EClassifier;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.util.ExtendedMetaData;

import com.ibm.ccl.mapping.xsd.XSDEcoreExtendedMetaData;
import com.ibm.ccl.mapping.xsd.XSDEcoreVisitor;


public class NamespaceVisitor extends XSDEcoreVisitor
{
  Map namespaceToEPackageMap = new HashMap();
  EPackage ePackage = null;
  List alreadyVisited = new ArrayList();
  
  public NamespaceVisitor()
  {
  }

  public void visitSchemas(List ePackages)
  {
	if (ePackages != null)
	{
	  for (Iterator i=ePackages.iterator(); i.hasNext();)	
	  {
	    ePackage = (EPackage)i.next();
	    alreadyVisited.clear();
   	    visitSchema(ePackage);
	  }
	}
  }
 
  public Map getNamespaceToEPackageMap()
  {
  	return namespaceToEPackageMap;
  }
  
  public void visitSchema(EPackage epackage)
  {
	String namespace = XSDEcoreExtendedMetaData.INSTANCE.getNamespace(epackage);
	if ((namespace != null) && (namespace.length()>0))
	{
	  if (!namespaceToEPackageMap.containsKey(namespace))
		namespaceToEPackageMap.put(namespace, epackage);
	}
	super.visitSchema(epackage);
  } 
  
  public void visitElement(EStructuralFeature element)
  {
	if ((element != null) && (!alreadyVisited.contains(element)))
	{
	  alreadyVisited.add(element);
	  String namespace = XSDEcoreExtendedMetaData.INSTANCE.getNamespace(element);
	  if ((namespace != null) && (namespace.length()>0))
	  {
		if (!namespaceToEPackageMap.containsKey(namespace))
		{
	      EPackage epackage = XSDEcoreExtendedMetaData.INSTANCE.getPackage(namespace);
	      if (epackage != null)
	      {
			namespaceToEPackageMap.put(namespace, epackage);
	      }
		}
 	  }
	  
	  EClassifier eClassifier = element.getEType();
	  if (eClassifier != null)
	  {
		visitType(eClassifier);
	  }
	}
  }
  
  public void visitAttribute(EStructuralFeature attr)
  {   
	if (attr != null)
	{
	  String namespace = XSDEcoreExtendedMetaData.INSTANCE.getNamespace(attr);
	  if ((namespace != null) && (namespace.length()>0))
	  {
		if (!namespaceToEPackageMap.containsKey(namespace))
		{
		  EPackage epackage = XSDEcoreExtendedMetaData.INSTANCE.getPackage(namespace);
	      if (epackage != null)
	      {
			namespaceToEPackageMap.put(namespace, epackage);
	      }
		}
 	  }
	}
  }
  
  public void visitComplexType(EClass complexType)
  {
	if ((complexType != null) && (!alreadyVisited.contains(complexType)))
	{
	  alreadyVisited.add(complexType);
	  String namespace = XSDEcoreExtendedMetaData.INSTANCE.getNamespace(complexType);
	  if ((namespace != null) && (namespace.length()>0))
	  {
		if (!namespaceToEPackageMap.containsKey(namespace))
		{
		  EPackage epackage = complexType.getEPackage();
		  if (epackage != null)
		  {
			namespaceToEPackageMap.put(namespace, epackage);
		  }
		}
	  }
	
	  // visit the XSD elements
	  for (Iterator i = ExtendedMetaData.INSTANCE.getElements(complexType).iterator(); i.hasNext();)
	    visitElement((EStructuralFeature)i.next());

	  // visit the XSD attributes
	  for (Iterator i = ExtendedMetaData.INSTANCE.getAttributes(complexType).iterator(); i.hasNext();)
	    visitAttribute((EStructuralFeature)i.next());
	}
  }
}