/*******************************************************************************
 *  Licensed Materials - Property of IBM
 * (C) Copyright IBM Corp.  2001, 2006.  All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.ccl.mapping.xsd.util;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.StringTokenizer;

import org.eclipse.emf.ecore.EAnnotation;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.util.BasicExtendedMetaData;

import com.ibm.ccl.mapping.xsd.XSDEcoreExtendedMetaData;

/**
 * @author lauzond
 *
 */
public class XSDExtendedMetaDataImpl extends BasicExtendedMetaData implements XSDEcoreExtendedMetaData
{

  public XSDExtendedMetaDataImpl()
  {
    super();
  }

  public XSDExtendedMetaDataImpl(EPackage.Registry registry)
  {
    super(registry);
  }

  public XSDExtendedMetaDataImpl(String annotationURI, EPackage.Registry registry)
  {
    super(annotationURI, registry);
  }

  public XSDExtendedMetaDataImpl(String annotationURI, EPackage.Registry registry, Map annotationMap)
  {
  	super(annotationURI, registry, annotationMap);
  }
  
 
  public boolean isQualified(EStructuralFeature eFeature)
  {
  	boolean result = false;
    if (eFeature != null)
    {
    	result = this.getNamespace(eFeature) != null;
    }
    return result;
  }
  
  public List getSourceWildcards(EStructuralFeature eFeature)
  {
    EAnnotation eAnnotation = getAnnotation(eFeature, false);

    if (eAnnotation != null)
    {
      String wildcards = (String)eAnnotation.getDetails().get("wildcards");
      if (wildcards != null)
      {
        List result = new ArrayList();
        for (StringTokenizer stringTokenizer = new StringTokenizer(wildcards); stringTokenizer.hasMoreTokens(); )
        {
          String wildcard = stringTokenizer.nextToken();
          result.add(wildcard);
        }
        return result;
      }
    }
    return Collections.EMPTY_LIST;
  }
}
