/*******************************************************************************
 *  Licensed Materials - Property of IBM
 * (C) Copyright IBM Corp.  2001, 2006.  All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.ccl.mapping.xsd.util;
/*
 * Licensed Material - Property of IBM 
 * 5724-I66
 * (C) Copyright IBM Corporation 2005 - All Rights Reserved. 
 * Note to U.S. Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.xsd.XSDAttributeDeclaration;
import org.eclipse.xsd.XSDAttributeGroupDefinition;
import org.eclipse.xsd.XSDAttributeUse;
import org.eclipse.xsd.XSDComplexTypeContent;
import org.eclipse.xsd.XSDComplexTypeDefinition;
import org.eclipse.xsd.XSDElementDeclaration;
import org.eclipse.xsd.XSDFeature;
import org.eclipse.xsd.XSDModelGroup;
import org.eclipse.xsd.XSDModelGroupDefinition;
import org.eclipse.xsd.XSDNamedComponent;
import org.eclipse.xsd.XSDParticle;
import org.eclipse.xsd.XSDSchema;
import org.eclipse.xsd.XSDSimpleTypeDefinition;
import org.eclipse.xsd.XSDTypeDefinition;
import org.eclipse.xsd.util.XSDConstants;
import org.eclipse.xsd.util.XSDUtil;


/**
 * @author arawlins
 *
 * Collection of utility methods for dealing with navigation of the XSD model
 */
public class XSDUtils {
    public static final String LOCAL = "##local";                            //$NON-NLS-1$
    public static final String OTHER = "##other";                            //$NON-NLS-1$
    public static final String ANY = "##any";                                //$NON-NLS-1$
    public static final String TARGET_NAMESPACE = "##targetNamespace";       //$NON-NLS-1$
	public static final String MAX_OCCURS_UNBOUNDED_STR_VALUE = "unbounded"; //$NON-NLS-1$
	public static final String ATTRIBUTE_USAGE_OPTIONAL_VALUE = "optional";  //$NON-NLS-1$
	public static final String ATTRIBUTE_USAGE_REQUIRED_VALUE = "required";  //$NON-NLS-1$
	
    private static List advancedPrimitives;

    // A list of all supported XSD types.  Usually the user will not be presented with the full list, but 
    // rather with the xsd short list
    private static List supportedPrimitives = new ArrayList();
    static {
        supportedPrimitives.add("anyType"); //$NON-NLS-1$
        supportedPrimitives.add("anyURI"); //$NON-NLS-1$
        supportedPrimitives.add("base64Binary"); //$NON-NLS-1$
        supportedPrimitives.add("boolean"); //$NON-NLS-1$
        supportedPrimitives.add("byte"); //$NON-NLS-1$
        supportedPrimitives.add("date"); //$NON-NLS-1$
        supportedPrimitives.add("dateTime"); //$NON-NLS-1$
        supportedPrimitives.add("decimal"); //$NON-NLS-1$
        supportedPrimitives.add("double"); //$NON-NLS-1$
        supportedPrimitives.add("duration"); //$NON-NLS-1$
        supportedPrimitives.add("ENTITIES"); //$NON-NLS-1$
        supportedPrimitives.add("ENTITY"); //$NON-NLS-1$
        supportedPrimitives.add("float"); //$NON-NLS-1$
        supportedPrimitives.add("gDay"); //$NON-NLS-1$
        supportedPrimitives.add("gMonth"); //$NON-NLS-1$
        supportedPrimitives.add("gMonthDay"); //$NON-NLS-1$
        supportedPrimitives.add("gYear"); //$NON-NLS-1$
        supportedPrimitives.add("gYearMonth"); //$NON-NLS-1$
        supportedPrimitives.add("hexBinary"); //$NON-NLS-1$
        supportedPrimitives.add("ID"); //$NON-NLS-1$
        supportedPrimitives.add("IDREF"); //$NON-NLS-1$
        supportedPrimitives.add("IDREFS"); //$NON-NLS-1$
        supportedPrimitives.add("int"); //$NON-NLS-1$
        supportedPrimitives.add("integer"); //$NON-NLS-1$
        supportedPrimitives.add("language"); //$NON-NLS-1$
        supportedPrimitives.add("long"); //$NON-NLS-1$
        supportedPrimitives.add("Name"); //$NON-NLS-1$
        supportedPrimitives.add("NCName"); //$NON-NLS-1$
        supportedPrimitives.add("negativeInteger"); //$NON-NLS-1$
        supportedPrimitives.add("NMTOKEN"); //$NON-NLS-1$
        supportedPrimitives.add("NMTOKENS"); //$NON-NLS-1$, "NMTOKENS"); //$NON-NLS-1$ //$NON-NLS-2$
        supportedPrimitives.add("nonNegativeInteger"); //$NON-NLS-1$
        supportedPrimitives.add("nonPositiveInteger"); //$NON-NLS-1$
        supportedPrimitives.add("normalizedString"); //$NON-NLS-1$
        supportedPrimitives.add("NOTATION"); //$NON-NLS-1$
        supportedPrimitives.add("positiveInteger"); //$NON-NLS-1$
        supportedPrimitives.add("QName"); //$NON-NLS-1$
        supportedPrimitives.add("short"); //$NON-NLS-1$
        supportedPrimitives.add("string"); //$NON-NLS-1$
        supportedPrimitives.add("time"); //$NON-NLS-1$
        supportedPrimitives.add("token"); //$NON-NLS-1$
        supportedPrimitives.add("unsignedByte"); //$NON-NLS-1$
        supportedPrimitives.add("unsignedInt"); //$NON-NLS-1$
        supportedPrimitives.add("unsignedLong"); //$NON-NLS-1$
        supportedPrimitives.add("unsignedShort"); //$NON-NLS-1$
    }


	/**
	 * Retrieves all the root Data types defined in the schema including complex types, user-defined simple types and anonymous
	 * complex types.  If there's an anonymous complex type definition (from a root element declaration) then we return
	 * the element declaration's anonymous type.
	 * @param schema
	 * @return
	 */
	public static List getAllDataTypes(XSDSchema schema)
	{
		if (schema==null)
			return Collections.EMPTY_LIST;
		
		List bos = new ArrayList();
		
        EList contents = schema.getContents();
        // First try the easy approach -- if this XSD contains a type definition, that's our BO, 
        // return it.  This is the recommended path, and the way our tooling does things.
        for (Iterator i = contents.iterator(); i.hasNext();)
		{
			Object item = i.next();
			if (item instanceof XSDTypeDefinition)
				bos.add((XSDTypeDefinition) item);
		}
        
		// If we failed, we try a second pass, this time looking for an element
		// with an anonymous complex
		// type defined in line
		for (Iterator i = contents.iterator(); i.hasNext();)
		{
			Object item = i.next();
			if (item instanceof XSDElementDeclaration)
			{
				XSDElementDeclaration element = (XSDElementDeclaration) item;
				if (element.getAnonymousTypeDefinition() instanceof XSDComplexTypeDefinition)
				{
					bos.add(element.getAnonymousTypeDefinition());
				}
			}
		}
		return bos;
	}

	/**
	 * Return the BO for the given schema.  If the supplied schema contains multiple top level complex type
	 * definitions, an exception is thrown.
	 * @param schema - The schema containing the BO.
	 * @param boName - The local name of the BO to retrieve -- if null method returns first BO found.
	 * @return The XSDComplexTypeDefinition if one exists, null otherwise
	 */
	public static XSDComplexTypeDefinition getBO(XSDSchema schema, String boName) {
		if(schema == null)
			return null;
		
		boolean pickFirstFound = (boName==null || boName.length()<1);
		
        EList contents = schema.getContents();
        // First try the easy approach -- if this XSD contains a complex type definition, that's our BO, 
        // return it.  This is the recommended path, and the way our tooling does things.
        for(Iterator i = contents.iterator(); i.hasNext(); ) {
        	Object item = i.next();
        	if(item instanceof XSDComplexTypeDefinition)
        		if (pickFirstFound)
        			return (XSDComplexTypeDefinition) item;
        		else if ( boName.equals(getDisplayName((XSDComplexTypeDefinition)item)))
        			return (XSDComplexTypeDefinition) item;
        }
        
        // If we failed, we try a second pass, this time looking for an element with an anonymous complex
        // type defined in line
        for(Iterator i = contents.iterator(); i.hasNext(); ) {
        	Object item = i.next();
        	if(item instanceof XSDFeature) {
        		XSDFeature element = (XSDFeature) item;
        		if(getResolvedType(element) instanceof XSDComplexTypeDefinition && getResolvedType(element).getSchema() == schema) {
        			if (pickFirstFound)
        				return (XSDComplexTypeDefinition) getResolvedType(element);
        			else if (boName.equals(getDisplayName(element)))
        				return (XSDComplexTypeDefinition) getResolvedType(element);
        		}
        		
        	}
        }
        
        return null;
		
	}
	
	/**
	 * Given a BO (XSD Complex Type), return a list of the attributes 
	 * within the complexType.
	 * @param bo
	 * @return List of XSDAttributeDeclaration
	 */
	public static List getChildAttributes(XSDComplexTypeDefinition bo)
	{
		EList attrContents = bo.getAttributeContents();
		List attrs = new ArrayList();
		for (int i=0; i< attrContents.size(); i++)
		{
			Object next = attrContents.get(i);
			
			// Attribute contents may include actual attribute delcarations (wrapped in XSDAttributeUses) or
			// attribute group definitions, containing bundles of attributes
			if(next instanceof XSDAttributeUse) {
				attrs.add( ((XSDAttributeUse) next).getContent() );
				
			} else if (next instanceof XSDAttributeGroupDefinition) {
				
				// Add these attributes to the end of attrContents to be processed in turn
				XSDAttributeGroupDefinition attrGroup = (XSDAttributeGroupDefinition) next;
				if(attrGroup.getResolvedAttributeGroupDefinition() != null)
					attrContents.addAll(attrGroup.getResolvedAttributeGroupDefinition().getAttributeUses());
				
			}
		}
		return attrs;
	}
	
	/**
	 * Given a BO (XSD Complex Type), return a list of the XSDFeatures 
	 * within the complexType's modelgroup (sequence, choice, etc.)
	 * @param bo
	 * @return
	 */
	public static List getChildElements(XSDComplexTypeDefinition bo) 
	{
   		return XSDUtils.getChildElements( getModelGroup(bo) );
	}

	/**
	 * Given an XSDFeature, return a list of the XSDFeatures 
	 * within the complexType's modelgroup (sequence, choice, etc.)
	 * @param bo
	 * @return
	 */
	public static List getChildElements(XSDFeature elem) 
	{
		XSDComplexTypeDefinition cType = getResolvedComplexType(elem);
		return (cType != null) ? XSDUtils.getChildElements(cType) : Collections.EMPTY_LIST;
	}
	
	
	/**
	 * Given a Model group, return a list of the XSDFeatures 
	 * declared within.
	 * @param group
	 * @return 
	 */
	public static List getChildElements(XSDModelGroup group) 
	{
    	if(group == null)
    		return new ArrayList();
    	
		List children = new ArrayList();
    	for(Iterator i = group.getContents().iterator(); i.hasNext();) 
    	{
    		XSDParticle next = (XSDParticle) i.next();
    		if(next.getTerm() instanceof XSDFeature)
    			children.add((XSDFeature) next.getTerm());
    		else if (next.getTerm() instanceof XSDModelGroup)
    			children.addAll(getChildElements((XSDModelGroup) next.getTerm()));
    	}
    	return children;
	}
	
	/**
	 * General utility method for finding a name for an xsd component.  This method will handle
	 * mapping of xsd primitive names to their human readable counterparts, as well as resolving
	 * element references for attributes and walking up object hierarchies for anonymous types.  
	 * Basically this method should be used whenever a name is needed to minimize the risk of 
	 * having a name of 'null' -- though this method WILL return null if no name can be found.
	 * @param component
	 * @return
	 */
	public static String getDisplayName(XSDNamedComponent component) {
		
		if(component == null)
			return null;
		
		if(component instanceof XSDTypeDefinition)
			return getDisplayNameFromXSDType((XSDTypeDefinition) component);
		
		if (component instanceof XSDFeature) {
			XSDFeature feature = (XSDFeature) component;
			if(feature.getName() != null)
				return feature.getName();
			else if (feature.getResolvedFeature() != null && feature.getResolvedFeature().getName() != null)
				return feature.getResolvedFeature().getName();
		}
		
		return component.getName();
		
	}
	
	/**
	 * Try a variety of methods to get a human readable name for type.  In order, this method will
	 * - check whether type is null, and if so return null
	 * - check whether type is a restriction of a primitive type, if so return its parent's name
	 * - check whether type is a complex anonymous (un-named) inner type of a named element, and if so, return the element's name  
	 * - check whether type is a primitive type, and if so return a human-readable version of that type
	 * - check whether type is a named, non-primitive type, and if so, return its name
	 * @param xsdType
	 * @return
	 */
	public static String getDisplayNameFromXSDType(XSDTypeDefinition type) {
		return getDisplayNameFromXSDType(type, true);
	}
	
    /**
     * Given an XSD Complex Type Definition, return the model group containing
     * its child elements. 
     * @param element
     * @return
     */
    public static XSDModelGroup getModelGroup(XSDComplexTypeDefinition cType) 
    {
        XSDParticle particle = cType.getComplexType();
        
        // In cases where cType doesn't have a model group AND cType has a parent with a modelgroup, the
        // call above will rather unexpectedly give us cType's PARENT's model group, rather than the null we 
        // might expect.  We don't want that here, if the model group returned is null or belongs to someone
        // other than us, return null
        if (particle==null || particle.eContainer() != cType)
            return null;
        
        // get the model group
        Object particleContent = particle.getContent();
        XSDModelGroup group = null;
        
        if (particleContent instanceof XSDModelGroupDefinition)
            group = ((XSDModelGroupDefinition)particleContent).getResolvedModelGroupDefinition().getModelGroup();
        else if (particleContent instanceof XSDModelGroup)
            group = (XSDModelGroup)particleContent;
        
        if (group==null)
            return null;

        // if the content of the complex type is empty then the content
        // must be in the complexContent, ie. we're extending another BO.
        // if the group and the type are not in the same resource then
        // we are extending another BO and we don't want to show inherited
        // attributes.
        if ( group.getContents().isEmpty() || group.eResource() != cType.eResource())
        {
            // the complex type's particle content is empty, check the content of the complex
            // type itself
            XSDComplexTypeContent content = cType.getContent();
            if (content instanceof XSDParticle)
                group = (XSDModelGroup)((XSDParticle)content).getContent();
        }       

        return group;
    }

	/**
	 * Try a variety of methods to get a human readable name for type.  In order, this method will
	 * - check whether type is null, and if so return null
	 * - check whether type is a complex anonymous (un-named) inner type of a named element, and if so, return the element's name  
	 * - check whether type is a primitive type, and if so return a human-readable version of that type
	 * - check whether type is a named, non-primitive type, and if so, return its name
	 * - if returnPrimitiveParents is true, check whether type is a restriction of a primitive type, 
	 * if so return its parent's name
	 * @param xsdType
	 * @param returnPrimitiveParents if true, and if type is an anonymous restriction of an xsd primitive 
	 * type, this method will return the name of the parent primitive type.  If false, restrictions of 
	 * primitive types will not be treated differently from other types, and their container hierarchy will
	 * be walked, instead of their inheritance hierarchy.  
	 * @return
	 */
	public static String getDisplayNameFromXSDType(XSDTypeDefinition type, boolean returnPrimitiveParents) {		
		if(type == null)
			return null;
		
		
		
		// Does type have a name?  If not, walk up the container tree to try and find one
		if(type.getName() == null || type.getName().length() == 0) {
			
			// In the special case where type is a restriction on a primitive type, just return the parent's
			// name (which will either be a primitive itself, or a named simple type)
			if(returnPrimitiveParents && isRestrictedPrimitiveType(type)) {
				return getDisplayNameFromXSDType(type.getBaseType());
			}
			
			EObject container = type.eContainer();

			while(container != null) {
				if(container instanceof XSDNamedComponent && ((XSDNamedComponent) container).getName() != null) {
					return ((XSDNamedComponent) container).getName();
				}
				container = container.eContainer();
			}
			// Type doesn't have a name, or a container with a name, nothing useful
			return null;
		} else
			return type.getName();
	}
	
	/**
	 * Return a collection of all types referenced by the attributes (XSDFeatures) of source.  This
	 * method is non-recursive (i.e. the list only contains direct references, not references-of-references)
	 * Will not return null, but may return an empty set.  This will not return a BO reference if the file
	 * has been deleted (or just doesn't exist).  
	 * @param source  The complex type to examine for references
	 * @return a Collection of XSDComplexTypeDefinition instances -- no duplicates
	 */
	public static Collection getReferencedTypes(XSDComplexTypeDefinition source) {
		if(source == null)
			return Collections.EMPTY_SET;
		
		List results = new ArrayList();
		XSDFeature element = null;
		XSDComplexTypeDefinition elementType = null;
		for(Iterator i = getChildElements(source).iterator(); i.hasNext(); ) {
			element = (XSDFeature) i.next();
			elementType = getResolvedComplexType(element); 
			if(elementType != null && !results.contains(elementType) && !XSDUtil.isSchemaForSchemaNamespace(elementType.getTargetNamespace()))
				results.add(elementType);
		}
		return results;
	}

	/**
	 * Return a collection of all types referenced by the attributes (XSDFeatures) of source.  This
	 * method is non-recursive (i.e. the list only contains direct references, not references-of-references)
	 * Will not return null, but may return an empty set.  This will return a BO reference if the file
	 * has been deleted (or just doesn't exist).  
	 * @param source  The complex type to examine for references
	 * @return a Collection of XSDTypeDefinition (could be complex or simple type) instances -- no duplicates
	 */
	public static Collection getAllReferencedTypes(XSDComplexTypeDefinition source)
	{
		return getAllReferencedTypes(source, false);
	}

	/**
	 * Return a collection of all types referenced by the attributes (XSDFeatures) of source.  This
	 * method is non-recursive (i.e. the list only contains direct references, not references-of-references)
	 * Will not return null, but may return an empty set.  This will return a BO reference if the file
	 * has been deleted (or just doesn't exist).  
	 * @param source  The complex type to examine for references
	 * @param includeAnonymous if true, the returned list will include anonymous inlined types as well.  These
	 * are not technically "referenced", however it allows this method to be used as a way to get all non-primitive
	 * types used in any way by source 
	 * @return a Collection of XSDTypeDefinition (could be complex or simple type) instances -- no duplicates
	 */
	public static Collection getAllReferencedTypes(XSDComplexTypeDefinition source, boolean includeAnonymous)
	{
		if (source == null)
			return Collections.EMPTY_SET;

		List results = new ArrayList();
		XSDTypeDefinition elementType = null;
		for (Iterator i = getChildElements(source).iterator(); i.hasNext();)
		{
			XSDFeature next = (XSDFeature) i.next();
			elementType = getResolvedType(next);
			
			// Only add non-null, non-duplicate, non-primitive types.  If includeAnonymous is false, 
			// anonymous types should be filtered out as well
			if(	elementType != null && 
				!results.contains(elementType) && 
				!XSDUtil.isSchemaForSchemaNamespace(elementType.getTargetNamespace()) &&
				(includeAnonymous || elementType.eContainer() != next) )
				results.add(elementType);
		}
		return results;
	}

	/**
	 * Given an element, return its complex type, or null if it does not have a complex type.  This is 
	 * slightly more complicated than just calling getType() since an element may not have a type at all, it
	 * may reference another element.
	 * @param element
	 * @return
	 */
	public static XSDComplexTypeDefinition getResolvedComplexType(XSDFeature feature) {
		// The contents of this method have been adapted to the more general getResolvedType,
		// but this method is maintained for compatibility and convenience 
		XSDTypeDefinition resolvedType = getResolvedType(feature);
		if(resolvedType instanceof XSDComplexTypeDefinition)
			return (XSDComplexTypeDefinition) resolvedType;
		return null;
	}

	/**
	 * Given an element, return its type, or null if it does not have a type.  This is 
	 * slightly more complicated than just calling getType() since an element may not have 
	 * a type at all, it may reference another element.
	 * @param element
	 * @return
	 */
	public static XSDTypeDefinition getResolvedType(XSDFeature feature) {
						
		// Special case of elements referencing stale XSD complex types
		if (feature instanceof XSDElementDeclaration && ((XSDElementDeclaration) feature).getTypeDefinition() instanceof XSDComplexTypeDefinition) {
			
			XSDElementDeclaration element = (XSDElementDeclaration) feature;

			// We have a type, but types can be proxies, and proxies can become
			// stale if the referenced
			// type changes, so before we return it, re-resolve the proxy and
			// then return it
			XSDComplexTypeDefinition oldType = (XSDComplexTypeDefinition) element.getTypeDefinition();
			EObject newType = EcoreUtil.resolve(element.getTypeDefinition(), element);
			if (oldType != newType) {
			    // We only return the resolved type if the name and the namespace has not changed.  Changing the name
			    // and namespace is essentially an unresolved BO.
			    String oldName = oldType.getName();
			    String newName = ((XSDTypeDefinition)newType).getName();
			    String oldTNS = oldType.getTargetNamespace();
			    String newTNS = ((XSDTypeDefinition)newType).getTargetNamespace();
			    
			    if ( ((oldName==newName) || (oldName!=null && oldName.equals(newName))) &&
			          ((oldTNS==newTNS) || (oldTNS!=null && oldTNS.equals(newTNS)))  )
			        element.setTypeDefinition((XSDTypeDefinition) newType);
			}
			return element.getTypeDefinition();

		} else if (feature.getType() != null) {
			return feature.getType();
		} else if (feature.getResolvedFeature() != null && feature.getResolvedFeature().getType() != null) {
			// We reference another element
			return feature.getResolvedFeature().getType();
		} else {
			return null;
		}
	}
	
	/**
	 * Return the base type from which this type inherits - that is, all xsd types are
	 * either xsd:anyType or xsd:anySimpleType at the topmost level of inheritance, so return the second
	 * topmost level of type's inheritance.  The first specific type from which type inherits. 
	 * @param type
	 * @return
	 */
	public static XSDTypeDefinition getRootType(XSDTypeDefinition type) {
		if(type == null)
			return null;
		
		XSDTypeDefinition baseType = type.getBaseType();
		while(baseType != null && !XSDUtil.isAnySimpleType(baseType) && !XSDUtil.isAnyType(baseType)) {
			// walk one more step up the hierarchy
			type = baseType;
			baseType = type.getBaseType();
		}
		
		// Since baseType, type's immediate parent, broke the while condition, we know that type is now
		// as high up the tree as we want to be
		return type;
		
		
		
	}
	
	/**
	 * Given a schema, return the list of XSDSimpleTypeDefinitions found within it.  Will not return null,
	 * but may return an empty list
	 * @param schema
	 * @return
	 */
	public static List getUserDefinedSimpleTypes(XSDSchema schema) {
		if(schema == null)
			return Collections.EMPTY_LIST;
		
		List result = new ArrayList();
		for(Iterator i = schema.getContents().iterator(); i.hasNext(); ) {
			Object next = i.next();
			if(next instanceof XSDSimpleTypeDefinition)
				result.add((XSDSimpleTypeDefinition) next);
		}
		return result;
	}
	
    /**
     * Return the enclosing Complex Type definition.
     * @param component
     * @return
     */
    public static XSDComplexTypeDefinition getEnclosingTypeDefinition(EObject component)
    {
        if (component == null)
            return null;
        
        if (component instanceof XSDComplexTypeDefinition)
            return (XSDComplexTypeDefinition)component;

        return getEnclosingTypeDefinition(component.eContainer());
    }

	/**
	 * Return true iff type is a descendant of a primitive xsd type.  Will not return true for primitives
	 * themselves.
	 * @param type
	 * @return
	 */
	public static boolean isRestrictedPrimitiveType(XSDTypeDefinition type) {
		if(type instanceof XSDComplexTypeDefinition)
			return false;
		
		XSDTypeDefinition baseType = getRootType(type);
		return getAdvancedPrimitives().contains(baseType);
	}
	
    /**
     * This returns a list of XSDTypeDefinitions.  However where getPrimitives() 
     * returns the basic set supported by the editor, getAdvancedPrimitives returns every known XSD
     * primitive type.
     * @return
     */
	public static List getAdvancedPrimitives() {
        if(advancedPrimitives == null) {
            advancedPrimitives = new ArrayList();
        
            // Get the schema for schemas instance to use when resolving primitives
            XSDSchema schemaForSchemas = XSDUtil.getSchemaForSchema(XSDConstants.SCHEMA_FOR_SCHEMA_URI_2001);
            
            // Start adding the simple types using the supportedPrimitives list
            for(Iterator i = supportedPrimitives.iterator(); i.hasNext(); ) {
                String typeName = (String) i.next();
                
                XSDTypeDefinition type = schemaForSchemas.resolveSimpleTypeDefinition(typeName);
                advancedPrimitives.add(type);
            }

            // Return primitives in alpha order
            Collections.sort(advancedPrimitives, new Comparator() {

                public int compare(Object o1, Object o2) {
                    if(o1 == null || o2 == null || ((XSDTypeDefinition) o1).getName() == null)
                        return 0;
                    return ((XSDTypeDefinition) o1).getName().compareToIgnoreCase(((XSDTypeDefinition) o2).getName());
                }
            });
        }
        return advancedPrimitives;      
    }

	/**
	 * Return the Data Type from the given schema with the given name.  
	 * @param schema - The schema containing the type.
	 * @param boName - The local name of the type to retrieve -- if null method returns first type found.
	 * @return The XSDTypeDefinition if one exists, null otherwise
	 */
	public static XSDTypeDefinition getDataType(XSDSchema schema, String typeName) {
		if(schema == null)
			return null;
		
		boolean pickFirstFound = (typeName==null || typeName.length()<1);
		
        EList contents = schema.getTypeDefinitions();
        
        // First try the easy approach -- if this XSD contains a type definition with the right name, that's 
        // our type, return it.  This is the recommended path, and the way our tooling does things.
        for(Iterator i = contents.iterator(); i.hasNext(); ) {
        	Object item = i.next();
        	if(item instanceof XSDTypeDefinition)
        		if (pickFirstFound)
        			return (XSDTypeDefinition) item;
        		else if ( typeName.equals(getDisplayName((XSDTypeDefinition)item)))
        			return (XSDTypeDefinition) item;
        }
        
        // If we failed, we try a second pass, this time looking for an element with an anonymous 
        // type defined in line
        for(Iterator i = contents.iterator(); i.hasNext(); ) {
        	Object item = i.next();
        	if(item instanceof XSDFeature) {
        		XSDFeature element = (XSDFeature) item;
        		XSDTypeDefinition resolvedType = getResolvedType(element); 
        		if(resolvedType != null && resolvedType.getSchema() == schema) {
        			if (pickFirstFound)
        				return resolvedType;
        			else if (typeName.equals(getDisplayName(element)))
        				return resolvedType;
        		}        		
        	}
        }        
        return null;
		
	}
    
    public static XSDTypeDefinition getTypeDef(XSDSchema schema, String typeName) {
        if(schema == null)
            return null;
        
        boolean pickFirstFound = (typeName==null || typeName.length()<1);
        
        EList contents = schema.getContents();
        
        // First try the easy approach -- if this XSD contains a type definition with the right name, that's 
        // our type, return it.  This is the recommended path, and the way our tooling does things.
        for(Iterator i = contents.iterator(); i.hasNext(); ) {
            Object item = i.next();
            if(item instanceof XSDTypeDefinition)
                if (pickFirstFound)
                    return (XSDTypeDefinition) item;
                else if ( typeName.equals(getDisplayName((XSDTypeDefinition)item)))
                    return (XSDTypeDefinition) item;
        }
        
        return null;
    }

    public static XSDElementDeclaration getElementDecl(XSDSchema schema, String elemName) {
        if(schema == null)
            return null;
        
        boolean pickFirstFound = (elemName==null || elemName.length()<1);
        
        EList contents = schema.getContents();

        for(Iterator i = contents.iterator(); i.hasNext(); ) {
            Object item = i.next();
            if(item instanceof XSDElementDeclaration) {
                XSDElementDeclaration element = (XSDElementDeclaration) item;
                if (pickFirstFound)
                    return element;
                else if (elemName.equals(getDisplayName(element)))
                    return element;
            }
        }
        return null;
    }
    
    public static XSDAttributeDeclaration getAttributeDecl(XSDSchema schema, String attrName) {
        if(schema == null)
            return null;
        
        boolean pickFirstFound = (attrName==null || attrName.length()<1);
        
        EList contents = schema.getContents();

        for(Iterator i = contents.iterator(); i.hasNext(); ) {
            Object item = i.next();
            if(item instanceof XSDAttributeDeclaration) {
                XSDAttributeDeclaration element = (XSDAttributeDeclaration) item;
                if (pickFirstFound)
                    return element;
                else if (attrName.equals(getDisplayName(element)))
                    return element;
            }
        }
        return null;
    }
}
